@@ -262,10 +262,11 @@ class FileTransferHandler(FileTransferAttributes):
 
     def _process_send_file_chunk(self, packet):
         chunk_id, chunk, file_data, has_more = packet[1:5]
+        chunk_id = bytestostr(chunk_id)
         filelog("_process_send_file_chunk%s", (chunk_id, chunk, "%i bytes" % len(file_data), has_more))
         chunk_state = self.receive_chunks_in_progress.get(chunk_id)
         if not chunk_state:
-            filelog.error("Error: cannot find the file transfer id '%s'", nonl(bytestostr(chunk_id)))
+            filelog.error("Error: cannot find the file transfer id '%s'", nonl(chunk_id))
             self.send("ack-file-chunk", chunk_id, False, "file transfer id not found", chunk)
             return
         fd = chunk_state[1]
@@ -317,9 +318,7 @@ class FileTransferHandler(FileTransferAttributes):
         elapsed = monotonic_time()-start_time
         mimetype = bytestostr(mimetype)
         filelog("%i bytes received in %i chunks, took %ims", filesize, chunk, elapsed*1000)
-        t = start_thread(self.do_process_downloaded_file, "process-download", daemon=False,
-                         args=(filename, mimetype, printit, openit, filesize, options))
-        filelog("started process-download thread: %s", t)
+        self.process_downloaded_file(filename, mimetype, printit, openit, filesize, options)
 
     def accept_data(self, send_id, dtype, basefilename, printit, openit):
         #subclasses should check the flags,
@@ -343,6 +342,7 @@ class FileTransferHandler(FileTransferAttributes):
 
     def _process_send_file(self, packet):
         #the remote end is sending us a file
+        start = monotonic_time()
         basefilename, mimetype, printit, openit, filesize, file_data, options = packet[1:8]
         send_id = ""
         if len(packet)>=9:
@@ -384,7 +384,6 @@ class FileTransferHandler(FileTransferAttributes):
             return
         self.file_descriptors.add(fd)
         if chunk_id:
-            chunk_id = strtobytes(chunk_id)
             l = len(self.receive_chunks_in_progress)
             if l>=MAX_CONCURRENT_FILES:
                 self.send("ack-file-chunk", chunk_id, False, "too many file transfers in progress: %i" % l, 0)
@@ -422,7 +421,14 @@ class FileTransferHandler(FileTransferAttributes):
             os.write(fd, file_data)
         finally:
             os.close(fd)
-        self.do_process_downloaded_file(filename, mimetype, printit, openit, filesize, options)
+        self.transfer_progress_update(False, send_id, monotonic_time()-start, filesize, filesize, None)
+        self.process_downloaded_file(filename, mimetype, printit, openit, filesize, options)
+
+
+    def process_downloaded_file(self, filename, mimetype, printit, openit, filesize, options):
+        t = start_thread(self.do_process_downloaded_file, "process-download", daemon=False,
+                         args=(filename, mimetype, printit, openit, filesize, options))
+        filelog("started process-download thread: %s", t)
 
     def do_process_downloaded_file(self, filename, mimetype, printit, openit, filesize, options):
         filelog("do_process_downloaded_file%s", (filename, mimetype, printit, openit, filesize, options))
@@ -803,12 +809,12 @@ class FileTransferHandler(FileTransferAttributes):
         #send some more file data
         filelog("ack-file-chunk: %s", packet[1:])
         chunk_id, state, error_message, chunk = packet[1:5]
+        chunk_id = bytestostr(chunk_id)
         if not state:
             filelog.error("Error: remote end is cancelling the file transfer:")
             filelog.error(" %s", error_message)
             del self.send_chunks_in_progress[chunk_id]
             return
-        chunk_id = bytestostr(chunk_id)
         chunk_state = self.send_chunks_in_progress.get(chunk_id)
         if not chunk_state:
             filelog.error("Error: cannot find the file transfer id '%s'", nonl(chunk_id))
