@@ -11,7 +11,7 @@ from xpra.util import csv
 from xpra.os_util import (
     WIN32, OSX, POSIX,
     osexpand, getuid, getgid, get_username_for_uid,
-    is_Debian, is_arm,
+    is_Debian, is_Ubuntu, is_arm,
     )
 
 def warn(msg):
@@ -133,6 +133,10 @@ def detect_xvfb_command(conf_dir="/etc/xpra/", bin_dir=None,
     if is_arm():
         #arm struggles to launch Xdummy, so use Xvfb:
         return get_Xvfb_command()
+    if is_Ubuntu() or is_Debian():
+        #These distros do weird things and this can cause the real X11 server to crash
+        #see ticket #2834
+        return get_Xvfb_command()
 
     xorg_bin = get_xorg_bin()
     def Xorg_suid_check():
